#include "SliceAugmenter.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Augments a Slice XML file with bases and quality values from outside of the\n"
"CLR range. Bases and quality values outside the CLR range are added as\n"
"<TrimmedSeq> records to each read in the <ReadCollection>. Position (left\n"
"or right) is correctly set with respect to direction of read.\n"
"\n"
"If a .qual.idx file from getCoverage is available, it will be used. If a \n"
".seq.idx file from filter_seq.pl -index is available, it will be used.\n"
"\n"
"   Usage: augmentSlice [options] prefix slice.xml\n"
"\n"
"     <prefix>  Prefix to .seq and .qual files\n"
"     slice.xml Slice XML file\n"
 + RequestManager::getOptionHelp();

    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    SliceAugmenter::handleOptions(tf);
    SliceAugmenter manager;

    string prefix;
    
    // Read the input filename
    if ((prefix = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify a prefix", "augmentSlice");
    }

    manager.setPrefix(prefix);


    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an slice file", "augmentSlice");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
