#include "ContigOptimizer.hh"


ContigOptimizer::ContigOptimizer(bool verbose)
  : m_verbose(verbose)
{

}


ContigOptimizer::~ContigOptimizer()
{

}

void ContigOptimizer::tilt(Contig * contig, long gindex, bool doTiltLeft)
{
  char cons;
  int currentCQV = contig->calculateCQV(gindex, cons);

  ContigSequenceSlice & tilingSlice = contig->getTilingSlice(gindex);
  ContigSequenceSlice::iterator t;

  for (t =  tilingSlice.begin();
       t != tilingSlice.end();
       t++)
  {
    ContigSequence * currentRead = *t;
    if (currentRead->base(gindex) == '-')
    {
      int twiddleDistance = 0;

      if (doTiltLeft)
      {
        currentRead->rightTwiddle(gindex, twiddleDistance, '?');
      }
      else
      {
        currentRead->leftTwiddle(gindex, twiddleDistance, '?');
      }

      if (twiddleDistance)
      {
        int newCQV = contig->calculateCQV(gindex, cons);

        if (newCQV < currentCQV)
        {
          // Undo if the alignment worsened
          if (doTiltLeft)
          {
            currentRead->rightTwiddle(gindex, twiddleDistance, '?');
          }
          else
          {
            currentRead->leftTwiddle(gindex, twiddleDistance, '?');
          }
        }
        else
        {
          // cerr << ((doTiltLeft) ? "right" : "left")
          //     << " twiddle (" << twiddleDistance 
          //    << ") at " << gindex << " in r" << currentRead->getReadCount() 
          //     << " o: " << currentCQV << " n: " << newCQV << endl;
          currentCQV = newCQV;
        }
      }
    }
  }

  /*
  if (cons != '-')
  {
    for (t =  tilingSlice.begin();
         t != tilingSlice.end();
         t++)
    {
      ContigSequence * currentRead = *t;
      if (currentRead->base(gindex) == '-')
      {
        int twiddleDistance = 0;

        if (doTiltLeft)
        {
          currentRead->rightTwiddle(gindex, twiddleDistance, cons);
        }
        else
        {
          currentRead->leftTwiddle(gindex, twiddleDistance, cons);
        }

        if (twiddleDistance)
        {
          int newCQV = calculateCQV(gindex, cons);

          cerr << ((doTiltLeft) ? "right" : "left")
               << " twiddle (" << twiddleDistance 
               << ") at " << gindex << " in r" << currentRead->getReadCount() 
               << " o: " << currentCQV << " n: " << newCQV << endl;
          currentCQV = newCQV;
        }
      }
    }
  }
  else if (currentCQV != 2*depth) // cons == '-'
  {
    map<ContigSequence *, int> twiddleMap;

    cerr << "twiddle all" << endl;
    for (t =  tilingSlice.begin();
         t != tilingSlice.end();
         t++)
    {
      ContigSequence * currentRead = *t;
      if (currentRead->base(gindex) == '-')
      {
        int twiddleDistance = 0;
        if (doTiltLeft)
        {
          if (currentRead->rightTwiddle(gindex, twiddleDistance, '?'))
          {
            twiddleMap[currentRead] = twiddleDistance;
            cerr << "td: " << twiddleDistance << endl;
          }
        }
        else
        {
          if (currentRead->leftTwiddle(gindex, twiddleDistance, '?'))
          {
            twiddleMap[currentRead] = twiddleDistance;
          }
        }
      }
    }

    if (!twiddleMap.empty())
    {
      contig->recallCons();
      dumpAlignment();
      // some gaps were twiddled
      int newCQV = calculateCQV(gindex, cons);

      map<ContigSequence *, int>::iterator i;

      for (i =  twiddleMap.begin();
           i != twiddleMap.end();
           i++)
      {
        // Iteratively undo each twiddle 

        if (doTiltLeft)
        {
          i->first->rightTwiddle(gindex, i->second, '?');
        }
        else
        {
          i->first->leftTwiddle(gindex, i->second, '?');
        }

        int undoCQV = calculateCQV(gindex, cons);

        if (undoCQV > newCQV)
        {
          // leave it untwiddled (no change from original)
        }
        else
        {
          // Keep the twiddle (redo)
          if (doTiltLeft)
          {
            i->first->rightTwiddle(gindex, i->second, '?');
          }
          else
          {
            i->first->leftTwiddle(gindex, i->second, '?');
          }

          cerr << ((doTiltLeft) ? "right" : "left")
               << "twiddle (" << i->second
               << ") at " << gindex << " in r" << i->first->getReadCount() 
               << " o: " << currentCQV
               << " n: " << newCQV
               << " c: " << undoCQV << endl;
          newCQV = undoCQV;

          // mark that this twiddle has been kept
          i->second = 0;
        }
      }
    }
  }
  */
}




void ContigOptimizer::tiltLeft(Contig * contig)
{
  for (int gindex = 0; gindex < m_gindex; gindex++)
  {
    tilt(contig, gindex, true);
  }
}

void ContigOptimizer::tiltRight(Contig * contig)
{
  for (int gindex = m_gindex-1; gindex >= 0; gindex--)
  {
    tilt(contig, gindex, false);
  }
}


void ContigOptimizer::optimize(Contig * contig)
{
  m_gindex = contig->getWidth();

  if (m_verbose) { cerr << "Contig: " << contig->getName() << endl; }
 
  if (m_verbose) { cerr << "pre-optimize" << endl; }
  contig->recallCons();
//  contig->dumpAlignment();

  if (m_verbose) { cerr << "tilt right" << endl; }
  tiltRight(contig);
//  contig->dumpAlignment();

  if (m_verbose) { cerr << "tilt left" << endl; }
  tiltLeft(contig);
//  contig->dumpAlignment();

  if (m_verbose) { cerr << "stripGapSlices" << endl; }
  contig->stripGapSlices();

  if (m_verbose) { cerr << "stripSequenceEdgeGaps" << endl; }
  contig->stripSequenceEdgeGaps();

  if (m_verbose) { cerr << "final" << endl; }
  contig->recallCons();
// contig->dumpAlignment();
}

