#include "TilingManager.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Converts the consensus of a slice XML file into a tasm file suitable for upload\n"
"with timmy.\n"
"\n"
"   Usage: slice2tasm [options] slice.xml\n"
"\n"
"     slice.xml Slice XML file\n"
"\n"
"   slice2tasm Options\n"
"   -------------------\n"
"   -i|--asmbl_id <id>     Specify the id for the tasm record\n"
"   -I|--ca_id <id>        Specify the ca_contig_id\n"
"   -d|--method <method>   Specify the method string\n"
"   -n|--comment <comment> Specify the comment string\n"
"   -u|--user <username>   Specify the userid\n"
"   -T|--date <datestring> Specify the mod_date and ed_date\n"
"   -q                     Compute quality classes in tasm file\n"
 + RequestManager::getOptionHelp();


    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    TilingManager::handleOptions(tf);
    TilingManager manager;

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an input file", "slice2tasm");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
