#ifndef SLICEZIPPER_HH 
#define SLICEZIPPER_HH 1

#include "RequestManager.hh"
#include "ReadMerger.hh"
#include <vector>

using libSlice::SliceData;


class VectorSlice : public SliceData
{
public:
  VectorSlice(bool intraGap,
              bool isGap,
              const ReadManager & readManager,
              const char * index, 
              const char * gindex,
              const char * existing,
              long sliceRangeIndex=0,
              long sliceRangeGindex=0);
  ~VectorSlice();

  bool isIntraGap() const;
  bool isGap() const;
  void setGap(bool);

private:
  bool m_intraGap;
  bool m_isGap;
};

typedef list<VectorSlice *> SliceVector;
typedef map<int, int> GapMap;

class SliceZipper : public RequestManager
{
public:
  SliceZipper();
  ~SliceZipper();


  void addRequest(ConsensusRequest * request);
  void addReadData(ReadData * readData);


  void addSliceRange(long & index, long & gindex);
  void addSlice(SliceData * slice);
  int  endSlices();

  static void handleOptions(Foundation * tf);


private:
  void mergeSlice(VectorSlice * slice, 
                  const string & nucData,
                  const string & qualData,
                  const string & readData,
                  char refcons,
                  bool isGap);

  void handleRefIntraGaps(SliceData * reference,
                          const string & newids,
                          const char * refconsensus,
                          long gindex,
                          long coordinate);

  VectorSlice * createVectorSlice(bool intraGap,
                                  bool isGap,
                                  const string & nucData,
                                  const string & qualData,
                                  const string & readData,
                                  const char * consensus);

  void createGapValues(SliceData * slice, string & nuc, string & qual);
  void printSlice(VectorSlice * slice);
  void storeQuerySlice(VectorSlice * query);
  void handleQuerySlice(SliceData * query, 
                        const string & newids,
                        const char * consensus);

  void handleSliceMerge(SliceData * reference, 
                        const string & newids, 
                        const char * consensus);

  int getGapCount(bool isReference, int gpos, int pos);
  void loadGaps(const string & gaplist, GapMap & gapmap);
  void addGap(int gappos, GapMap & gapmap);

  SliceVector m_sliceResults;
  SliceVector::iterator m_sliceResultsLocation;

  long m_currentReferencePosition;
  long m_currentReferenceUngapped;
  long m_currentQueryPosition;
  long m_currentQueryUngapped;

  static int m_alignStart;
  static int m_promoteAlignmentGaps;

  long m_newIndex;
  long m_newGindex;

  long m_origRangeIndex;
  long m_origRangeGindex;
  int m_rangeCount;
  
  ReadMerger m_readMerger;
  ReadManager m_uberReadManager;

  static string m_referenceGaps;
  static string m_queryGaps;

  static int m_ungappedCoordinates;
  static int m_reverseQuery;

  static int m_gapCoordinateOffset;
  static int m_referenceConsensus;

  bool m_testReferenceGaps;
  bool m_mergedSlices;

  GapMap m_referenceGapMap;
  GapMap m_queryGapMap;

  SliceData * m_lastslice;
};

#endif
